/*
 * File: BSTMap.java
 * -----------------
 * This file defines a partial implementation of the Map
 * interface using a binary search tree as the underlying
 * representation.
 */

import java.io.*;

/* Class: BSTMap */
/**
 * This class provides a partial implementation of the
 * Map interface based on binary search trees.  For
 * simplicity, this implementation includes only the put
 * and get methods and allows only strings as keys.  The
 * defining feature of a binary search tree is that the
 * key value in its root node is greater than the keys
 * in its left subtree and less than those to the right.
 */
public class BSTMap {

/* Constructor: BSTMap() */
/**
 * Creates an empty binary search tree.
 */
	public BSTMap() {
		root = null;
	}

/* Method: put(key, value) */
/**
 * Creates a binding between the specified key and the
 * associated value. If a node for this key already
 * exists in this tree, the new value replaces the old one.
 * If the key is not present, the put method creates a
 * new node at the appropriate position in the tree.
 */
	public void put(String key, Object value) {
		if (root == null) {
			root = new BSTNode();
			root.key = key;
			root.value = value;
			root.left = new BSTMap();
			root.right = new BSTMap();
		} else {
			int cmp = key.compareTo(root.key);
			if (cmp == 0) {
				root.value = value;
			} else if (cmp < 0) {
				root.left.put(key, value);
			} else {
				root.right.put(key, value);
			}
		}
	}

/* Method: get(key) */
/**
 * Returns the binding associated with the specified key.
 * If no value has been puted, get returns null.
 */
	public Object get(String key) {
		if (root == null) {
			return null;
		} else {
			int cmp = key.compareTo(root.key);
			if (cmp == 0) {
				return root.value;
			} else if (cmp < 0) {
				return root.left.get(key);
			} else {
				return root.right.get(key);
			}
		}
	}

/* Method: listBindings(out) */
/**
 * Lists the bindings in this map in alphabetical order,
 * using the specified PrintStream to display the output.
 */
	public void listBindings(PrintStream out) {
		if (root != null) {
			root.left.listBindings(out);
			out.println(root.key + " = " + root.value);
			root.right.listBindings(out);
		}
	}

/* Method: listBindings(wr) */
/**
 * Lists the bindings in this map in alphabetical order,
 * using the specified PrintWriter to display the output.
 */
	public void listBindings(PrintWriter wr) {
		if (root != null) {
			root.left.listBindings(wr);
			wr.println(root.key + " = " + root.value);
			root.right.listBindings(wr);
		}
	}

/* Private instance variables */
	private BSTNode root;
}

/* Private class: BSTNode */
/**
 * This class provides the structure definition necessary
 * to store individual nodes in the binary search tree.
 */
class BSTNode {
	String key;
	Object value;
	BSTMap left, right;
}
