/*
 * File: CLString.java
 * -------------------
 * This file defines a new implementation of String that
 * represents its underlying data as a character list.
 */

/* Class: CLString */
/**
 * This class provides string operations similar to those
 * in the standard String class, but with a different
 * underlying representation.  That representation
 * happens to be a linked list of characters, but
 * clients need not be aware of this fact.
 */
public class CLString {

/* Constructor: CLString(str) */
/**
 * Creates a new CLString from a standard Java string.
 */
	public CLString(String str) {
		list = toCLString(str);
	}

/* Method: toString() */
/**
 * Converts a CLString to a standard Java string.
 */
	public String toString() {
		return toString(list);
	}

/* Method: length() */
/**
 * Returns the number of characters in the CLString.
 */
	public int length() {
		return length(list);
	}

/* Method: concat(s2) */
/**
 * Concatenates a second CLString to the end of this one.
 */
	public CLString concat(CLString s2) {
		return new CLString(concat(list, s2.list));
	}

/* Private constructor: CLString(list) */
/**
 * Creates a new CLString from a character list.
 */
	private CLString(CharCell list) {
		this.list = list;
	}

/* Private method: toCLString(str) */
/**
 * Creates a character list corresponding to the string.
 */
	private CharCell toCLString(String str) {
		if (str.length() == 0) {
			return null;
		} else {
			CharCell list = new CharCell();
			list.ch = str.charAt(0);
			list.link = toCLString(str.substring(1));
			return list;
		}
	}

/* Private method: toString(list) */
/**
 * Converts the character list to a string.
 */
	private String toString(CharCell list) {
		if (list == null) {
			return "";
		} else {
			return list.ch + toString(list.link);
		}
	}

/* Private method: length(list) */
/**
 * Calculates the length of the character list.
 */
	private int length(CharCell list) {
		if (list == null) {
			return 0;
		} else {
			return 1 + length(list.link);
		}
	}

/* Private method: concat(s1, s2) */
/**
 * Creates a new list by concatenating s1 and s2.
 */
	private CharCell concat(CharCell s1, CharCell s2) {
		if (s1 == null) {
			return s2;
		} else {
			CharCell list = new CharCell();
			list.ch = s1.ch;
			list.link = concat(s1.link, s2);
			return list;
		}
	}

/* Private state */

	private CharCell list;

}

/* Private class: CharCell */
/**
 * This class provides the structure definition necessary
 * to store individual cells in a character list.
 */
class CharCell {
	char ch;
	CharCell link;
}
