/*
 * File: Compound.java
 * -------------------
 * This file defines the expression subclass consisting
 * of an integer constant.
 */

import java.io.*;

/* Class: Compound */
/**
 * This expression subclass represents a compound
 * expression consisting of an operator and two
 * subexpressions.
 */
public abstract class Compound extends Expression {

/* Constructor: Compound(op, lhs, rhs) */
/**
 * Creates a compound expression that combines the two
 * operands with the specified operator.
 */
	public Compound(char op, Expression lhs, Expression rhs) {
		this.op = op;
		this.lhs = lhs;
		this.rhs = rhs;
	}

/* Method: eval() */
/**
 * Evaluates an expression to return an integer.  For
 * the Compound class, the result is defined in terms
 * of the apply method, which differs for each operator.
 */
	public int eval() {
		return apply(lhs.eval(), rhs.eval());
	}

/* Method: unparse(wr) */
/**
 * Writes out a human-readable version of the expression
 * to the specified PrintWriter.  For a Compound, this
 * method is implemented recursively.
 */
	public void unparse(PrintWriter wr) {
		wr.print("(");
		lhs.unparse(wr);
		wr.print(" " + op + " ");
		rhs.unparse(wr);
		wr.print(")");
	}

/* Abstract method: apply(lhs, rhs) */
/**
 * Implements the meaning of a specific operator.  Each
 * Compound subclass will define the apply method to
 * carry out the computation for that operator.
 */
	public abstract int apply(int lhs, int rhs);

/* Private state */
	private char op;
	private Expression lhs, rhs;

}
