/*
 * File: Expression.java
 * ---------------------
 * This file defines the abstract Expression class.
 */

import java.io.*;

/* Class: Expression */
/**
 * This abstract class specifies the behavior that all
 * objects of type Expression must implement.  Because
 * Expression is an abstract class, only the concrete
 * subclasses of Expression can actually be constructed.
 */
public abstract class Expression {

/* Default constructor: Expression */
/**
 * Creates a default expression.  Java requires that
 * this empty constructor exist so that the subclass
 * constructors can create the common superclass.
 */
	protected Expression() { }

/* Abstract method: eval() */
/**
 * Evaluates an expression to return an integer.  Each
 * Expression subclass must implement this method.
 */
	public abstract int eval();

/* Abstract method: unparse(wr) */
/**
 * Writes out a human-readable version of the expression
 * to the specified PrintWriter.
 */
	public abstract void unparse(PrintWriter wr);

}
